// Unreal Fighting Framework by Kevin Poretti

#pragma once

#include "CoreMinimal.h"
#include "GameFramework/GameState.h"
#include "FFGameState.generated.h"

constexpr float ONE_TICK = 0.0083333333;
constexpr int64 TICKS_PER_SECOND = 120;

/**
 * 
 */
UCLASS()
class UNREALFIGHTINGFRAMEWORK_API AFFGameState : public AGameState
{
	GENERATED_BODY()

public:

    UFUNCTION(BlueprintPure)
	int64 GetCurrentTick() const { return CurrentTick; }

    virtual void OnFixedTick(float OneTick);

	// Begin AActor interface
	virtual void Tick(float DeltaSeconds) override;
	// End AActor interface
private:
    /**
	 * Amount of time accumulated from ticks. When Tick is called the delta time since the
	 * last Tick will be added to this variable.
	 *
	 * Once enough time has accumulated to simulate at least one frame (defined by the value ONE_TICK) then the
	 * game logic will update/tick. ONE_TICK's worth of time will be subtracted from the this variable
	 */
	float AccumulatedTime = 0;

	int64 CurrentTick = 0;
};
